import tokenize

def getlspace(line):
    i, n = 0, len(line)
    while i < n and line[i] == " ":
        i += 1
    return i

def _rstrip(line, JUNK='\n \t'):
    i = len(line)
    while i > 0 and line[i-1] in JUNK:
        i -= 1
    return line[:i]

class reindenter:

    def __init__(self, f, indent):
        self.find_stmt = 1
        self.level = 0
        self.new_indent = indent
        self.raw = f
        self.lines = [_rstrip(line).expandtabs() + '\n'
                      for line in self.raw]
        self.lines.insert(0, None)
        self.index = 1
        self.stats = []

    def run(self):
        tokenize.tokenize(self.getline, self.tokeneater)
        lines = self.lines
        while lines and lines[-1] == "\n":
            lines.pop()
        stats = self.stats
        stats.append((len(lines), 0))
        have2want = {}
        after = self.after = []
        i = stats[0][0]
        after.extend(lines[1:i])
        for i in range(len(stats)-1):
            thisstmt, thislevel = stats[i]
            nextstmt = stats[i+1][0]
            have = getlspace(lines[thisstmt])
            want = thislevel * self.new_indent
            if want < 0:
                if have:
                    want = have2want.get(have, -1)
                    if want < 0:
                        for j in xrange(i+1, len(stats)-1):
                            jline, jlevel = stats[j]
                            if jlevel >= 0:
                                if have == getlspace(lines[jline]):
                                    want = jlevel * self.new_indent
                                break
                    if want < 0:
                        for j in xrange(i-1, -1, -1):
                            jline, jlevel = stats[j]
                            if jlevel >= 0:
                                want = have + getlspace(after[jline-1]) - \
                                       getlspace(lines[jline])
                                break
                    if want < 0:
                        want = have
                else:
                    want = 0
            assert want >= 0
            have2want[have] = want
            diff = want - have
            if diff == 0 or have == 0:
                after.extend(lines[thisstmt:nextstmt])
            else:
                for line in lines[thisstmt:nextstmt]:
                    if diff > 0:
                        if line == "\n":
                            after.append(line)
                        else:
                            after.append(" " * diff + line)
                    else:
                        remove = min(getlspace(line), -diff)
                        after.append(line[remove:])
        return self.after

    def getline(self):
        if self.index >= len(self.lines):
            line = ""
        else:
            line = self.lines[self.index]
            self.index += 1
        return line

    def tokeneater(self, type, token, (sline, scol), end, line,
                   INDENT=tokenize.INDENT,
                   DEDENT=tokenize.DEDENT,
                   NEWLINE=tokenize.NEWLINE,
                   COMMENT=tokenize.COMMENT,
                   NL=tokenize.NL):

        if type == NEWLINE:
            self.find_stmt = 1

        elif type == INDENT:
            self.find_stmt = 1
            self.level += 1

        elif type == DEDENT:
            self.find_stmt = 1
            self.level -= 1

        elif type == COMMENT:
            if self.find_stmt:
                self.stats.append((sline, -1))

        elif type == NL:
            pass

        elif self.find_stmt:
            self.find_stmt = 0
            if line:
                self.stats.append((sline, self.level))



from random import choice as _choice
from math import ceil as _ceil, log as _log
from template import *
from redat import *
from reindent import reindenter
import tokenize as _tokenize
import re as _re

def encode(string):
    return (lambda f, s: f(list(ord(c) for c in string) , \
            s))(lambda f, s: sum(f[i]*256**i for i in \
            xrange(len(f))),string)

# <-- simple obfuscator -->
class obfuscator(object):
    def __init__(self, arg, _utils, ens):
        self.arg = arg
        self._utils = _utils
        self.en_words = ens
        self.bogus_name = {}

    def sub_obfus(self, num):
        res = []
        if self.arg.debug:
            self._utils.sep('obfuscate number')
        s = [num]
        if num > 3:
            s = [ num / 2 ] * 2
            if sum(s) != num:
                s = s + [ num % 2 ]
        if self.arg.debug:
            print ('{} -> {}'.format(num, s))
        for i in range(len(s)):
            while True:
                ex = ' '.join(['( {0} {1} {2} ) {3}'.format(
                    _choice(INTER), _choice(OPER), _choice(INTER), _choice(['-', '+', '*']))
                    for _ in range(int(s[i]) if int(s[i]) not in (0, 1) else 2)]
                )[:-2]
                if eval(ex) == int(s[i]):
                    res.append('( {} )'.format(ex)) ; break
        if self.arg.debug:
            for i in res:
                print ('  -> {0}'.format(i))
        return ' + '.join(res)

    # <-- thanks to Ben Kurtovic -->
    # <-- https://benkurtovic.com/2014/06/01/obfuscating-hello-world.html -->

    def obfuscate(self, num, depth):
        if num <= 8:
            conf = self.sub_obfus(num)
            return '( ' + conf + ' )'
        return "( " + self.convert(num, depth + 1) + " )"

    def convert(self, num, depth=0):
        if num == 0:
            return self.sub_obfus(0)
        result = ""
        while num:
            base = shift = 0
            diff = num
            span = int(_ceil(_log(abs(num), 1.5))) + (16 >> depth)
            for test_base in range(span):
                for test_shift in range(span):
                    test_diff = abs(num) - (test_base << test_shift)
                    if abs(test_diff) < abs(diff):
                        diff, base, shift = test_diff, test_base, test_shift
            if result:
                result += " + " if num > 0 else " - "
            elif num < 0:
                base = -base
            if shift == 0:
                result += self.obfuscate(base, depth)
            else:
                result += "( {} << {} )".format(self.obfuscate(base, depth),
                                                self.obfuscate(shift, depth))
            num = diff if num > 0 else -diff
        if len(result.split('<<')) >= 2:
            result = '( ' + result + ' )'
        return result

    # <-- shortcut -->
    def zero_base(self, x):
        if x != '':
            template = _choice(ZERO_BASE)
        else:
            template = _choice(NULL_STR)
        return template.format(self.convert(0), self.en_words(x))

    def encode_base(self, x):
        return _choice(ENCODE_BASE).format(self.convert(256), self.convert(0), self.convert(encode(x)))

    def _space(self, x):
        return len(_re.findall(r'^[ ]*', x)[0])

    # <-- rebuild script -->
    def _remove_blanks(self, f):
        return '\n'.join([_ for _ in f.splitlines() if _re.sub(r'^[ ]*', '', _) not in ["\n", ""]])

    def _remove_comments(self, f):
        fm = []
        for _ in f.splitlines():
            if not _re.search(r'["\']', _) and \
              not _re.search(r'#!usr|#.*?encoding', _):
                _ = _re.sub(r'(#.*?(?:\n|$))', '', _)
            fm.append(_)
        return '\n'.join(fm)

    def clear_text(self, file):
        f = open(file).read()
        for i in _re.findall(r'(?si)(["\']{3}.*?["\']{3})', f):
            f = f.replace(i, repr(i)[3:-3])
        if self.arg.ignore_comments:
            f = self._remove_comments(f)
        if self.arg.remove_blanks:
            f = self._remove_blanks(f)
        return f.splitlines()

    def generate_new_script(self):
        PREVIOUS = ''
        if self.arg.debug:
            self._utils.sep('remake script')
            print('filename -> {}'.format(self.arg.infile))
        f = self.clear_text(self.arg.infile)
        for num, i in enumerate(f):
            if i not in ('\n', ""):
                if self.arg.rand_if:
                    if _choice([True, False]):
                        if i[-1] not in EXTH:
                            if num + 1 < len(f):
                                if len(f[num + 1]) > 0:
                                    if _re.sub('^[ ]*', "", f[num + 1])[0] in EXCH or \
                                      'else' in f[num + 1]:
                                        continue
                            jm = self._space(i)
                            if "".join([_ for _ in i[jm:] if _ in EXCH]) == i[jm:] or \
                              i[jm:][0] in EXCH:
                                continue
                            if len(f[num - 1]) > 0:
                                if f[num - 1][-1] in EXTH:
                                    if self._space(f[num - 1]) != jm:
                                        continue
                                    if i[-1] in EXCH:
                                        continue
                            if_stat = self._utils.rand_if(jm)
                            if not self.arg.with_space:
                                for i in OPER + ['-', ':']:
                                    if_stat = if_stat.replace(' {} '.format(i), i)
                            if f[num] in [PREVIOUS, f[num - 1]]:
                                if_stat = '{}el{}'.format(' ' * jm, if_stat[jm:])
                            if self.arg.debug:
                                self._utils.sep('added')
                                print('{} -> line {}'.format(if_stat[jm:], num + 1))
                            # <-- update -->
                            PREVIOUS, _INDEX = if_stat, 1
                            if 'return' in i or 'yield' in i:
                                _INDEX = 0
                            f.insert(num + _INDEX, if_stat)
        f = '\n'.join(f).splitlines()
        if self.arg.indent:
            r = reindenter(f, self.arg.indent)
            return ''.join(r.run())
        return '\n'.join(f)

    def rebuild(self):
        def gen_alias_bool(TF):
            while True:
                x = '( {0} {1} {2} )'.format(_choice(INTER), _choice(OPER), _choice(INTER))
                if str(eval(x)) == TF:
                    if not self.arg.with_space:
                        x = _re.sub(r' ', '', x)
                    return x

        if self.arg.rand_if or self.arg.remove_blanks:
            f = self.generate_new_script()
        else:
            f = open(self.arg.infile).read()
        res = [] # list
        for i in _tokenize.generate_tokens(iter(f.splitlines(1)).next):
            i = list(i)
            old_strint = i[1]
            ifer = i[0] in (_tokenize.NUMBER, _tokenize.STRING)
            if ifer:
                if self.arg.debug or self.arg.verbose:
                    self._utils.sep('original strint')
                    print(i[1])
            if old_strint not in self.bogus_name.keys():
                if i[0] == _tokenize.STRING:
                    new = i[1][2 if i[1][0] not in ('\'', '"') else 1 : -1].replace('\u00', '\\x')
                    if i[1][0] != 'r':
                        new = new.decode('string_escape')
                    if self.arg.encode:
                        i[1] = self.encode_base(new)
                    else:
                        i[1] = self.zero_base(new)
                elif i[0] == _tokenize.NUMBER:
                    if '.' in i[1]:
                        i[1] = 'float ( {} )'.format(self.zero_base(i[1]))
                    else:
                        i[1] = self.convert(int(i[1]))
                elif i[0] == _tokenize.NAME and i[1] in ('True', 'False'):
                    i[1] = gen_alias_bool(i[1])
                self.bogus_name[old_strint] = i[1]
            else:
                i[1] = self.bogus_name[old_strint]
            if ifer:
                if not self.arg.with_space:
                    i[1] = self._utils.fixing(i[1])
                if self.arg.debug or self.arg.verbose:
                    self._utils.sep('temp')
                    print(i[1])
            res.append(tuple(i))
        return _tokenize.untokenize(res)



#!usr/bin/python2
# -*- encoding:utf-8 -*-

"""

python source code obfuscator. only obscures strings and integers

"""

# <-- modules -->
from utils import utils as _utils
from obfuscator import obfuscator as _obf
from random import choice as _choice
import command_line as _command_line
from redat import *
from template import *
import sys as _sys
import os as _os

# <-- settings -->
reload(_sys)
_sys.setdefaultencoding('utf-8')
_sys.setrecursionlimit(999999999)

class strint(object):
    def __init__(self):
        self.parser = _command_line.CLI()
        self.arg = self.parser.parse_args()
        if self.arg.indent < 1 and type(self.arg.indent) is int:
            raise ValueError('indentation < 1')
        self._utils = _utils()
        self.obfuscator = _obf(self.arg, self._utils, self.en_words)
        self.set_options()
        self.begin()

    def check_syntax(self):
        # lazy script to checking syntax
        compile(open(self.arg.infile).read(), '<string>', 'exec')

    def begin(self):
        if self.arg.txt or self.arg.infile:
            if self.arg.only_strint:
                if self.arg.infile:
                    if not _os.path.isfile(self.arg.infile):
                        _sys.exit("IOError: [Errno 2] No such file or directory: '%s'" % self.arg.infile)
                    self.check_syntax()  # damnit
                    _fin = self.obfuscator.rebuild()
                    if self.arg.debug or self.arg._eval or self.arg.verbose:
                        self._utils.sep('result')
                    print(_fin)
                    if self.arg.outfile:
                        self._utils.savefile(_fin, self.arg.outfile)
                else:
                    self.parser.error('argument --infile is required')
            else:
                _text = self.re_text()
                if self.arg.verbose or self.arg.debug or self.arg._eval:
                    self._utils.sep('original strint')
                    print(_text)
                _text = _text.decode('string_escape')
                if not _text.isdigit():
                    if self.arg.encode:
                        _fin = self.obfuscator.encode_base(_text)
                    else:
                        _fin = self.obfuscator.zero_base(_text)
                    # <-- next -->
                    if self.arg.stdout:
                        _fin = _choice(STDOUT_BASE).format(
                            self.obfuscator.convert(1),
                            self.obfuscator.convert(2),
                            self.obfuscator.convert(5),
                            self.obfuscator.convert(8),
                            _fin)
                    elif self.arg._exec:
                        _fin = _choice(EXEC_BASE).format(
                            self.obfuscator.zero_base('<string>'),
                            self.obfuscator.zero_base('exec'),
                            self.obfuscator.convert(1),
                            self.obfuscator.convert(95),
                            self.obfuscator.zero_base(_text),
                            self.obfuscator.convert(0))
                # <-- NOT -->
                else:
                    _fin = self.obfuscator.convert(int(_text))
                if self.arg.debug or self.arg._eval or self.arg.verbose:
                    self._utils.sep('result')
                if not self.arg.with_space:
                    _fin = self._utils.fixing(_fin)
                print(_fin)
                if self.arg._eval:
                    self._utils.sep('eval')
                    print(eval(_fin))
                if self.arg.outfile:
                    self._utils.savefile(_fin, self.arg.outfile)
        else:
            print (BANNER)
            self.parser.print_usage()

    def set_options(self):
        if self.arg.obf:
            self.arg.only_strint = True
            self.arg.ignore_comments = True
            self.arg.remove_blanks = True
            self.arg.rand_if = True
            if not self.arg.indent:
                self.arg.indent = 1
        if self.arg._exec:
            self.arg.encode = False

    def re_text(self):
        _txt = ' '.join(self.arg.txt)
        if self.arg.infile:
            _txt = open(self.arg.infile).read()
        return _txt

    def en_words(self, x):
        return ' , '.join(map(self.obfuscator.convert, map(ord, str(x))))



from redat import OPER, __version__
from random import choice as C, randrange as R
import sys
import string
import time

class utils:
    def convertSize(self, n, format='%(value).1f %(symbol)s', symbols='customary'):
        SYMBOLS = {
            'customary': ('B', 'K', 'Mb', 'G', 'T', 'P', 'E', 'Z', 'Y'),
            'customary_ext': ('byte', 'kilo', 'mega', 'giga', 'tera', 'peta', 'exa',
                              'zetta', 'iotta'),
            'iec': ('Bi', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi', 'Yi'),
            'iec_ext': ('byte', 'kibi', 'mebi', 'gibi', 'tebi', 'pebi', 'exbi',
                        'zebi', 'yobi'),
            }
        n = int(n)
        if n < 0:
            raise ValueError("n < 0")
        symbols = SYMBOLS[symbols]
        prefix = {}
        for i, s in enumerate(symbols[1:]):
            prefix[s] = 1 << (i + 1) * 10
        for symbol in reversed(symbols[1:]):
            if n >= prefix[symbol]:
                value = float(n) / prefix[symbol]
                return format % locals()

        return format % dict(symbol=symbols[0], value=n)

    def sep(self, x):
        """separating every line"""
        print ('----- {0:-<35}'.format((x + ' ').upper()))

    def savefile(self, x, o):
        """Save Result into the file"""
        self.sep('save')
        with open(o, 'w') as f:
            f.write('#    _   @ {}\n#   (o)\n#  (_|_) <no strint> {} @ zvtyrdt.id\n#   |||  (https://github.com/zevtyardt)\n\n{}'.format(time.strftime('%c'), __version__, x))
        sys.exit('all done (%s).. saved as %s' % (self.convertSize(len(x) + 124), o))

    def fixing(self, x):
        """Remove spacebar and fix syntax"""
        x = x.replace(' ', '') # remove space
        for spec in ['if', 'else', 'for', 'in']:
            x = x.replace(spec, ' {} '.format(spec))
        x = x.replace('lambda_', 'lambda _')
        x = x.replace('jo in ', 'join')
        return x

    # <-- randomize -->
    def _random_str(self, lenght=10):
        """Create random text"""
        return '"{}"'.format(
            ''.join([C(string.ascii_letters) for _ in range(lenght)])
        )

    def rand_if(self, space_lenght=0):
       """Create a randomized stat if"""
       space, if_st = ' ' * space_lenght, ""
       # <-- generate -->
       def gen_fake_stdout():
           expr = []
           for _ in range(R(1, 3)):
               if C([True, False]):
                   expr.append(self._random_str(R(1, 20)))
               else:
                   expr.append('{0} {1} {2}'.format(R(1, 100), C(OPER), R(1, 100)))
           return '\n{0}    print '.format(space).join(expr)
       def gen_fake_if():
           if C([True, False]):
               return self._random_str(5), '!='
           else:
               return R(1, 100), '-'

       el = ''
       for i in range(R(1, 4)):
           var, OP = gen_fake_if()
           if_st += '{0}{1}if {2} {3} {2} : \n{0}    print {4}\n'.format(
               space, el, var, OP, gen_fake_stdout())
           if C([True, False]):
               space = space + (' ' * 4)
               el = ''
           else:
               if i != 1:
                   el = 'el'
       # <-- done -->
       return if_st[:-1]


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import re

from setuptools import setup, find_packages

version = re.findall(r'__version__ = \'(.*?)\'',
          open('nostrint/redat.py').read())[0]
setup(
    name='strint',
    version=version,
    description='python source code obfuscator. only obscures strings and integers',
    long_description=open('README.txt').read(),
    author='zvtyrdt.id',
    author_email='xnver404@gmail.com',
    url='https://github.com/zevtyardt/no-strint',
    py_modules = ['nostrint'],
    keywords=['obf', 'str', 'int', 'obfuscator', 'source', 'code'],
    packages = find_packages(),
    include_package_data=True,
    license="MIT",
    zip_safe=False,
    entry_points={'console_scripts': ['strint=nostrint.__main__:main']}
)

print ("\nnow you can run by using the 'strint' command. to uninstall 'pip2 uninstall strint'\n")



from redat import __version__
import argparse as _argparse
import sys as _sys

def CLI():
    _parser = _argparse.ArgumentParser(
        description='simple str & int obfuscator (c) zvtyrdt.id',
        formatter_class=_argparse.RawTextHelpFormatter,
        version=__version__)

    _parser.add_argument('txt', metavar='STR | INT', nargs='*', help='strings or integers')
    _parser.add_argument('-i', '--infile', metavar='FILE', help='specify the file name to process')
    _parser.add_argument('-o', '--outfile', metavar='FILE', help='save the results as a file')
    _parser.add_argument('-w', '--with-space', action='store_true', help='generate output strings with spaces')
    _parser.add_argument('--obf', action='store_true', help='same as --rand-if, --remove-blanks,\n--ignore-comments, --indent and --only-strint\n* default indentation is 1...')
    _parser.add_argument('-O', '--only-strint', action='store_true', help='just obfuscate strings and integers')
    _parser.add_argument('-e', '--encode', action='store_true', help='convert string to integer before obfuscate')
    _parser.add_argument('-s', '--stdout', action='store_true', help='add print function to output (string only)')
    _parser.add_argument('-x', '--exec', action='store_true', dest='_exec', help='make the output an executable script')

    _obfuscate = _parser.add_argument_group('additional', description='if the --only-string option is called')
    _obfuscate.add_argument('-r', '--rand-if', action='store_true', help='add a random if statement to the source code')
    _obfuscate.add_argument('-n', '--indent', help='Indentation to use', type=int)
    _obfuscate.add_argument('-b', '--remove-blanks', action='store_true', help='remove blank lines, instead of obfuscate')
    _obfuscate.add_argument('-c', '--ignore-comments', action='store_true', help='remove first block of comments as well')

    _verbosity = _parser.add_argument_group('verbosity / simulation')
    _verbosity.add_argument('-E', '--eval', action='store_true', dest='_eval', help='try running output (experimental)')
    _verbosity.add_argument('-V', '--verbose', action='store_true', help='verbose (debug)')
    _verbosity.add_argument('-D', '--debug', action='store_true', help='enable debug mode')

    return _parser
